package edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.connection.ConnectionFWFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.TransactionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.constants.ErrorConstants;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;

/**
 * Classe que ens servirà per la gestió de les transaccions
 * @author Llorenç Camps
 * 12/12/2013
 */
public class TransactionFWImpl implements TransactionFW{

	private static final Logger log = Logger.getLogger( TransactionFWImpl.class );
	
	private Connection connection;
	
	private static boolean DEFAULT_AUTOCOMMIT = true;
	
	// Constructor
	
	public TransactionFWImpl ( boolean autoCommit ) throws DBException {
		try {
			// Obtenim la connexió
			connection = ConnectionFWFactory.getInstance().getConnection();
		} catch (DBException e) {
			log.error( "Error obtenint la connexió" );
			throw e;
		}
		try {
			// Setejem l'autocommit
			connection.setAutoCommit( autoCommit );
		} catch (SQLException e) {
			log.error( "Error setejant l'autoCommit a [" + autoCommit + "]. " + e.getMessage() );
			throw new DBException( ErrorConstants.ERROR_TRANSACTION_CONNECTION_CODI, ErrorConstants.ERROR_TRANSACTION_CONNECTION_MSG + autoCommit, e );
		}
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.TransactionFW#commit()
	 */
	@Override
	public void commit() throws DBException {
		if( this.connection == null ){
			throw new DBException( ErrorConstants.ERROR_CONNECTION_NULL_CODI, ErrorConstants.ERROR_CONNECTION_NULL_MSG );
		}
		try {
			this.connection.commit();
		} catch (SQLException e) {
			log.error( "Error fent commit. " + e.getMessage() );
			throw new DBException( ErrorConstants.ERROR_COMMIT_CODI, ErrorConstants.ERROR_COMMIT_MSG, e );
		}
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.TransactionFW#rollback()
	 */
	@Override
	public void rollback() throws DBException {
		if( this.connection == null ){
			throw new DBException( ErrorConstants.ERROR_CONNECTION_NULL_CODI, ErrorConstants.ERROR_CONNECTION_NULL_MSG );
		}
		try {
			this.connection.rollback();
		} catch (SQLException e) {
			log.error( "Error fent rollback. " + e.getMessage() );
			throw new DBException( ErrorConstants.ERROR_ROLLBACK_CODI, ErrorConstants.ERROR_ROLLBACK_MSG, e );
		}		
	}
	
	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.TransactionFW#close()
	 */
	@Override
	public void close() throws DBException {
		try {			
			// Abans de fer el close modifiquem l'autocommit de la connexió al valor per defecte (true)
			if( !this.connection.getAutoCommit() ) 
				this.connection.setAutoCommit( DEFAULT_AUTOCOMMIT );
			if( this.connection != null )
				this.connection.close();			
		} catch (SQLException e) {
			log.error( "Error fent el close. " + e.getMessage() );
			throw new DBException( ErrorConstants.ERROR_CLOSE_CODI, ErrorConstants.ERROR_CLOSE_MSG, e );
		}finally{
			this.connection = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.TransactionFW#executeQuery(java.lang.String)
	 */
	@Override
	public void executeQuery(String query) throws DBException {
		try {
			Statement st = this.connection.createStatement();
			if( st.executeUpdate( query ) != 1 ){
				throw new DBException( ErrorConstants.ERROR_UPDATE_MULTIPLE_RECORDS_CODI, ErrorConstants.ERROR_UPDATE_MULTIPLE_RECORDS_MSG );
			}
		} catch (SQLException e) {
			log.error( "Error executant la [query:" + query + "]");
			throw new DBException( ErrorConstants.ERROR_QUERY_UPDATE_CODI, ErrorConstants.ERROR_GENERIC_DATABASE_MSG + e.getMessage(), e );
		}
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.TransactionFW#executeQueryData(java.lang.String)
	 */
	@Override
	public ResultSet executeQueryData(String query) throws DBException {
		if( this.connection == null ){
			throw new DBException( ErrorConstants.ERROR_CONNECTION_NULL_CODI, ErrorConstants.ERROR_CONNECTION_NULL_MSG );
		}
		try {
			Statement st = this.connection.createStatement();
			return st.executeQuery( query );
		} catch (SQLException e) {
			log.error( "Error executant la [query:" + query + "]");
			throw new DBException( ErrorConstants.ERROR_QUERY_EXECUTE_CODI, ErrorConstants.ERROR_GENERIC_DATABASE_MSG + e.getMessage(), e );
		}
	}

	/*
	 * (non-Javadoc)
	 * @see edu.uoc.pfc.lcamps.persistenciaFW.admin.transaction.TransactionFW#executeCount(java.lang.String)
	 */
	@Override
	public int executeCount(String query) throws DBException {
		if( this.connection == null ){
			throw new DBException( ErrorConstants.ERROR_CONNECTION_NULL_CODI, ErrorConstants.ERROR_CONNECTION_NULL_MSG );
		}
		try{
			Statement st = this.connection.createStatement();
			ResultSet rs = st.executeQuery( query );
			// Ens movem a la primera posició del resultset 
			rs.next();
			return rs.getInt(1);
		} catch (SQLException e) {
			log.error( "Error executant la [query:" + query + "]");
			throw new DBException( ErrorConstants.ERROR_QUERY_COUNT_CODI, ErrorConstants.ERROR_GENERIC_DATABASE_MSG + e.getMessage(), e );
		}
	}
}