package edu.uoc.pfc.lcamps.webapp.catwings.bd.dao;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFW;
import edu.uoc.pfc.lcamps.persistenciaFW.admin.session.SessionFWFactory;
import edu.uoc.pfc.lcamps.persistenciaFW.exception.DBException;

/**
 * Classe abstracta que ens servirà per gestionar les principals operacions del framework de persistència.
 * Totes les classes DAO extendran d'aquesta aixi dispondran de les operacions 
 * @author Llorenç Camps 
 */
public abstract class AbstractDAO<E> {
	
	private static final Logger log = Logger.getLogger( AbstractDAO.class );
	
	/**
	 * Constructor
	 */
	public AbstractDAO(){super();}
	
	/**
	 * Obrim una nova transacció
	 * @param autocommit
	 * @return
	 * @throws DBException si produeix algun error amb la Base de dades.
	 */
	protected SessionFW openTransaction(boolean autocommit) throws DBException{
	  log.debug("openTransaction() - Inici[autocommit:" + autocommit + "]" );
	  SessionFW sessio = SessionFWFactory.getInstance().createSessionFW();
	  sessio.newTransaction( autocommit );
	  log.debug("openTransaction() - Fi" );
	  return sessio;
	}
	
	/**
	 * Realitza un close de la transacció
	 * @param sessio
	 * @throws DBException si produeix algun error amb la Base de dades.
	 */
	protected void closeTransaction(SessionFW sessio) throws DBException{
	  sessio.closeTransaction();
	  log.debug("closeTransaction() - Fi" );
	}
	
	/**
	 * Realitza una cerca a on retorna només un únic element.
	 * @param obj
	 * @return
	 * @throws DBException si produeix algun error amb la Base de dades.
	 */
	@SuppressWarnings("unchecked")
	protected E findSingle(E obj) throws DBException {
	  log.debug(" findSingle() - init");
	  if( null != obj ) log.debug(" findSingle() - [obj:" + obj.toString() + "]");
		
	  SessionFW sessio = SessionFWFactory.getInstance().createSessionFW();
	  sessio.newTransaction();		
		
	  List<E> out = (List<E>) sessio.findListFromObj( obj );
		
	  log.debug(" findSingle() - init");
	  if( out == null) return null;
	  return out.get(0);
	}
	
	/**
	 * Realitza una cerca a on retorna un llistat d'objectes
	 * @param obj
	 * @return
	 * @throws DBException si produeix algun error amb la Base de dades.
	 */
	@SuppressWarnings("unchecked")
	protected List<E> findList(E obj) throws DBException{
	  log.debug(" findList() - init");
	  if( null != obj ) log.debug(" findList() - [obj:" + obj.toString() + "]");
	  SessionFW sessio = SessionFWFactory.getInstance().createSessionFW();
	  sessio.newTransaction();
		  
	  List<E> out =(List<E>) sessio.findListFromObj( obj );

	  log.debug(" findList() - fi");
	  if( out == null) return new ArrayList<E>();
	  return out;
	}
	
	/**
	 * Realitza una cerca passant una query i on retorna un llistat d'objectes
	 * @param sql
	 * @param obj
	 * @return
	 * @throws DBException si produeix algun error amb la Base de dades.
	 */
	@SuppressWarnings("unchecked")
	protected List<E> findList(String sql, E obj) throws DBException{
	  log.debug(" findList() - init");
	  if( null != obj ) log.debug(" findList() - [sql:" + sql + ", obj:" + obj.toString() + "]");
	  SessionFW sessio = SessionFWFactory.getInstance().createSessionFW();
	  sessio.newTransaction();
		  
	  List<E> out =(List<E>) sessio.findListFromQuery( sql, obj );

	  log.debug(" findList() - fi");
	  if( out == null) return new ArrayList<E>();
	  return out;
	}
	
	/**
	 * Realitza la insercio de l'objecte
	 * @param sessio
	 * @param obj
	 * @return
	 * @throws DBException si produeix algun error amb la Base de dades.
	 */
	protected int insert(SessionFW sessio, E obj) throws DBException{
	  log.debug(" insert() - init [obj:" + obj.toString() + "]");
	  int id = sessio.insert( obj );
	  log.debug(" insert() - fi [id:" + id + "]" );
	  return id;
	}
	
	/**
	 * Realitza la actualització de l'objecte
	 * @param sessio
	 * @param obj
	 * @return
	 * @throws DBException si produeix algun error amb la Base de dades.
	 */
	protected void update(SessionFW sessio, E obj) throws DBException{
	  log.debug(" update() - init [obj:" + obj.toString() + "]");
	  sessio.update( obj );
	  log.debug(" update() - fi" );
	}
	
	/**
	 * Realitza l'esborrat de l'objecte
	 * @param sessio
	 * @param obj
	 * @return
	 * @throws DBException si produeix algun error amb la Base de dades.
	 */
	protected void delete(SessionFW sessio, E obj) throws DBException{
	  log.debug(" delete() - init [obj:" + obj.toString() + "]");
	  sessio.delete( obj );
	  log.debug(" delete() - fi" );
	}

}